--------------------------------------------------------------------------------
-- Weapon Plasma Gun + Projectile plasma by Vectar666
-- Based on:
-- Weapon Laser + Projectile Ray
-- Original Carnage Contest Weapon
-- Script by DC, August 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.d3==nil then cc.d3={} end
cc.d3.plasmagun={}
cc.d3.plasmagun.plasma={}

-- Load & Prepare Ressources
cc.d3.plasmagun.gfx_wpn=loadgfx("weapons/d3_plasmagun.bmp")					-- Weapon Image
setmidhandle(cc.d3.plasmagun.gfx_wpn)
cc.d3.plasmagun.gfx_pro=loadgfx("weapons/d3_plasma.bmp")				-- Projectile Image
setmidhandle(cc.d3.plasmagun.gfx_pro)
cc.d3.plasmagun.sfx_attack=loadsfx("d3_plasmagun.wav")						-- Attack Sound
cc.d3.plasmagun.sfx_impact=loadsfx("d3_plasma_impact.wav")					-- Impact Sound

--------------------------------------------------------------------------------
-- Weapon: Laser
--------------------------------------------------------------------------------

cc.d3.plasmagun.id=addweapon("cc.d3.plasmagun","Series 3 Plasma Gun",cc.d3.plasmagun.gfx_wpn,0,2)	-- Add Weapon
cc.d3.plasmagun.ammo=30

function cc.d3.plasmagun.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.d3.plasmagun.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.d3.plasmagun.ammo-weapon_shots>0 then
		hudammobar(cc.d3.plasmagun.ammo-weapon_shots,cc.d3.plasmagun.ammo)
	end
	-- HUD Crosshair
	if cc.d3.plasmagun.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.d3.plasmagun.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.d3.plasmagun.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Reset Timer
		math.randomseed(12345*weapon_shots*getframe()*666)
		weapon_timer=math.random(5,6)
		-- Attack
		playsound(cc.d3.plasmagun.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.d3.plasmagun.plasma.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*13.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*13.0
		-- Timer
		projectiles[id].timer=0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.d3.plasmagun.plasma.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlecolor(0,55,255)
		particlefadealpha(0.01)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		if (weapon_shots>=cc.d3.plasmagun.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Ray
--------------------------------------------------------------------------------

cc.d3.plasmagun.plasma.id=addprojectile("cc.d3.plasmagun.plasma")			-- Add Projectile

function cc.d3.plasmagun.plasma.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(0,55,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.d3.plasmagun.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.d3.plasmagun.plasma.update(id)						-- Update
	-- Move
	cc.d3.plasmagun.plasma.move(id)
end

function cc.d3.plasmagun.plasma.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>500 then
		-- Free projectile
		freeprojectile(id)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/2)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				if playercollision()~=0 then
					playerpush(playercollision(),projectiles[id].sx/17.0,projectiles[id].sy/17.0)
					playerdamage(playercollision(),16)
					blood(projectiles[id].x+math.sin(math.rad(rot))*20,projectiles[id].y-math.cos(math.rad(rot))*20)
				elseif objectcollision()>0 then
					objectdamage(objectcollision(),16)
				end
				-- Destroy terrain
				local gray=math.random(0,20)
				terrainalphaimage(gfx_crater25,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21,math.random(6,10)*0.1,gray,gray,gray)
				terraincircle(projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21,5,0x00000000)
				-- Effects
				playsound(cc.d3.plasmagun.sfx_impact)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*21,projectiles[id].y-math.cos(math.rad(rot))*21)
				particlecolor(0,55,255)
				particlefadealpha(0.01)
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			playsound(cc.d3.plasmagun.sfx_impact)
			particle(p_smoke,projectiles[id].x,projectiles[id].y)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end