--------------------------------------------------------------------------------
-- Weapon  D3 Rocket Launcher by Vectar666
-- Based on:
-- Weapon Plasma Blaster + Projectile Ray
-- Original Carnage Contest Weapon
-- Script by DC, December 2009, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
if cc.d3==nil then cc.d3={} end
cc.d3.rckt={}
cc.d3.rckt.rocket={}

-- Load & Prepare Ressources
cc.d3.rckt.gfx_wpn=loadgfx("weapons/d3_rckt.bmp")			-- Weapon Image
sethandle(cc.d3.rckt.gfx_wpn,10,12)
cc.d3.rckt.gfx_pro=loadgfx("weapons/d3_rcktpro.bmp")				-- Projectile Image
setmidhandle(cc.d3.rckt.gfx_pro)
cc.d3.rckt.sfx_attack=loadsfx("d3_rckt.ogg")						-- Attack Sound
cc.d3.rckt.sfx_impact=loadsfx("d3_rcktexplode.wav")				-- Impact Sound

--------------------------------------------------------------------------------
-- Weapon: Plasma Blaster
--------------------------------------------------------------------------------

cc.d3.rckt.id=addweapon("cc.d3.rckt","Rocket Launcher",cc.d3.rckt.gfx_wpn,0,2)	-- Add Weapon (0 uses, first in round 3)
cc.d3.rckt.ammo=5													-- 5 Shots

function cc.d3.rckt.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.d3.rckt.gfx_wpn,8,0)
	-- HUD ammobar
	if cc.d3.rckt.ammo-weapon_shots>0 then
		hudammobar(cc.d3.rckt.ammo-weapon_shots,cc.d3.rckt.ammo)
	end
	-- HUD Crosshair
	if cc.d3.rckt.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.d3.rckt.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.d3.rckt.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Reset Timer
		weapon_timer=40
		-- Attack
		playsound(cc.d3.rckt.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.d3.rckt.rocket.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*8.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*8.0
		-- Timer
		projectiles[id].timer=0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.d3.rckt.rocket.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(4)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlecolor(255,200,0)
		particlefadealpha(0.01)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		if (weapon_shots>=cc.d3.rckt.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Rocket
--------------------------------------------------------------------------------

cc.d3.rckt.rocket.id=addprojectile("cc.d3.rckt.rocket")			-- Add Projectile

function cc.d3.rckt.rocket.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(0.9,0.9)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.d3.rckt.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.d3.rckt.rocket.update(id)							-- Update
	-- Particle Tail
	particle(p_lightpuff,projectiles[id].x,projectiles[id].y)
	particlespeed(math.random(-5,5)*0.1,math.random(-5,5)*0.1)
	particlefadealpha(0.05)
	particlesize(math.random(20,40)*0.1,math.random(20,40)*0.1)
	particlecolor(255,200,0)
	-- Move
	cc.d3.rckt.rocket.move(id)
end

function cc.d3.rckt.rocket.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>500 then
		-- Free projectile
		freeprojectile(id)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Cause damage
				arealdamage(projectiles[id].x,projectiles[id].y,40,125)
				-- Destroy terrain
				terrainexplosion(projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5,37,2)
				quake()
				-- Crater
				grey=math.random(0,20)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(7,10)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(7,10)*0.1,grey,grey,grey)
				end
				-- Effects
				playsound(cc.d3.rckt.sfx_impact)
				particle(p_smoke,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
				particlefadealpha(0.006)
				particle(p_muzzle,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
				particlefadealpha(1.2)
				particlesize(3.5,3.5)
				particlecolor(255,200,0)
				particlefadealpha(0.005)
				for j=1,30,1 do
					particle(p_lightpuff,projectiles[id].x+math.sin(math.rad(rot))*15,projectiles[id].y-math.cos(math.rad(rot))*15)
					particlespeed(math.random(-20,20)*0.2,math.random(-20,20)*0.2)
					particlefadealpha(0.03)
					particlesize(math.random(40,70)*0.15,math.random(40,70)*0.15)
					particlecolor(255,200,0)
				end
				for i=1,5 do
					particle(p_spark,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)
					particlecolor(255,200,0)
					particlefadealpha(0.01)
					particlesize(2.5,2.5)
				end
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end